
/* ------------------------------------------------------------------------- */

/*

macro.h

United States Naval Observatory
USNO/NRL Optical Interferometer
3450 Massachusetts Avenue, NW
Washington, DC  20392-5420

Subsystem:
----------
My software libraries

Description:
------------
This is the macro header file.

Modification history:
---------------------
1992 Jul 27 - Nicholas Elias, USNO/AD5
	      File created.

*/

/* ------------------------------------------------------------------------- */

/* Define the macros for the simple mathematical functions */

#define CONVFLOAT( Var )   ( (float) ( Var ) )
#define CONVDOUBLE( Var )   ( (double) ( Var ) )

#define SWAP( TYPE, Var1, Var2 ) \
    { \
      TYPE Temp = ( Var1 ); \
      ( Var1 ) = ( Var2 ); \
      ( Var2 ) = Temp; \
    }

#define DELTA( Var1, Var2 )   ( Var1 == Var2 ? +1 : 0 )
#define NOTDELTA( Var1, Var2 )   ( Var1 != Var2 ? +1 : 0 )

#define SIGN( Var1, Var2 )   ( Var2 >= 0.0 ? fabs( Var1 ) : - fabs( Var1 ) )
#define SIGNUM( Var )   ( Var > 0 ? +1 : Var < 0 ? -1 : 0 )

#define MODULO( Var1, Var2, VarMod ) \
    { \
      double dfVar; \
      dfVar = (double) Var1; \
      while ( dfVar >= fabs( Var2 ) ) dfVar -= fabs( Var2 ); \
      VarMod = dfVar; \
    }

#define ROUND( Var )   ( (int) floor( ( (double) Var ) + 0.5 ) )
#define TRUNC( Var )   ( (int) floor( (double) Var ) )

#define PYTHAG( Var1, Var2, Pythag ) \
    { \
      if ( ( fabs( Var1 ) == 0.0 ) && ( fabs( Var2 ) == 0.0 ) ) \
      { \
        Pythag = 0.0; \
      } \
      else \
      { \
        if ( fabs( Var1 ) > fabs( Var2 ) ) \
        { \
          Pythag = fabs( Var1 ) * sqrt( 1.0 + pow( ( Var2 / Var1 ), 2.0 ) ); \
        } \
        else \
        { \
          Pythag = fabs( Var2 ) * sqrt( 1.0 + pow( ( Var1 / Var2 ), 2.0 ) ); \
        } \
      } \
    }

#define MAX( Var1, Var2 )   ( Var1 > Var2 ? Var1 : Var2 )
#define MIN( Var1, Var2 )   ( Var1 < Var2 ? Var1 : Var2 )

#define MAXDATA( iNumData, pData, Max ) \
    { \
      int iData; \
      Max = *pData; \
      for ( iData = 1; iData <= ( iNumData - 1 ); iData++ ) \
      { \
        Max = MAX( Max, *( pData + iData ) ); \
      } \
    }
#define MINDATA( iNumData, pData, Min ) \
    { \
      int iData; \
      Min = *pData; \
      for ( iData = 1; iData <= ( iNumData - 1 ); iData++ ) \
      { \
        Min = MIN( Min, *( pData + iData ) ); \
      } \
    }

#define MEAN( iNumData, pData, Mean ) \
    { \
      int iData; \
      Mean = 0.0; \
      for ( iData = 0; iData <= ( iNumData - 1 ); iData++ ) \
      { \
        Mean += *( pData + iData ); \
      } \
      Mean /= ( (double) iNumData ); \
    }

#define WMEAN( iNumData, pData, pDataErr, WMean ) \
    { \
      int iData; \
      double Temp = 0.0; \
      WMean = 0.0; \
      for ( iData = 0; iData <= ( iNumData - 1 ); iData++ ) \
      { \
	if ( *( pDataErr + iData ) > 0.0 ) \
	{ \
          WMean += *( pData + iData ) \
	      / ( *( pDataErr + iData ) * *( pDataErr + iData ) ); \
          Temp += 1.0 / ( *( pDataErr + iData ) * *( pDataErr + iData ) ); \
        } \
      } \
      if ( Temp > 0.0 ) \
      { \
        WMean /= Temp; \
      } \
    }

#define STDDEV( iNumData, pData, Mean, StdDev ) \
    { \
      int iData; \
      if ( iNumData < 2 ) \
      { \
	Mean = 0.0; \
        StdDev = -1.0; \
      } \
      else \
      { \
        StdDev = 0.0; \
        for ( iData = 0; iData <= ( iNumData - 1 ); iData++ ) \
        { \
          StdDev += ( *( pData + iData ) - Mean ) \
	      * ( *( pData + iData ) - Mean ); \
        } \
	if ( iNumData > 1 ) \
	{ \
          StdDev /= ( (double) ( iNumData - 1 ) ); \
        } \
        StdDev = sqrt( StdDev ); \
      } \
    }

#define MUSIGMA( iNumData, pData, Mu, Sigma ) \
    { \
      MEAN( iNumData, pData, Mu ); \
      STDDEV( iNumData, pData, Mu, Sigma ); \
      if ( ( iNumData > 2 ) && ( Sigma > 0.0 ) ) \
      { \
        Sigma /= sqrt( (double) iNumData ); \
      } \
    }


/* Define the macros for the simple unit conversion functions */

#define DECCODEC( Angle )   ( 90 - Angle )

#define HRS2DEGS( Hrs )   ( 15.0 * Hrs )
#define DEGS2HRS( Degs )   ( Degs / 15.0 )

#define HRS2RADS( Hrs )   ( ( ( 2.0 * PI / 360.0 ) * 15 ) * Hrs )
#define RADS2HRS( Rads )   ( ( 1.0 / 15.0 ) * ( 360.0 / ( 2.0 * PI ) ) * Rads )

#define DEGS2RADS( Degs )   ( ( ( 2.0 * PI ) / 360.0 ) * Degs )
#define RADS2DEGS( Rads )   ( ( 360.0 / ( 2.0 * PI ) ) * Rads )

