/* define the PGPLOT driver and the initialization command */
#include <tcl.h>
#include <string.h>

/* 
 *  This code is a bit platform dependent as it makes some assumptions
 *  about how FORTRAN will name this tcltkdriver
 *
 */

/* Global variables shared between initpg and the driver */

static Tcl_Interp *interpreter = NULL;
static char canvasid[80] = ".pgplot";
static int X_WID = 0 ;
static int Y_WID = 0 ;

void SaveInterp(Tcl_Interp *interp) {
   interpreter = interp;
}

/*
 *  NCOLOR -- number of colors
 *
 *	PGPLOT predefines 16 colors, we can probably allow more later
 */

#define NCOLOR	16
/*  The pre-defined dark grey color is too dark (?) */

static unsigned char R_table[NCOLOR] = 
{  1,255,255,  0,  0,  0,255,255, 255,  0,  0,  0,127,255, 85,170};
static unsigned char G_table[NCOLOR] =
{  1,255,  0,255,  0,255,  0,255, 127,142,255,127,  0,  0, 85,170};
static unsigned char B_table[NCOLOR] =
{  1,255,  0,  0,255,255,255,  0,   0,  0,127,255,255,127, 85,170};

/*
 *  Misc static variables
 */

static int COLOR = 1 ;			/* current pen color */
static int APPEND = 0 ;			/* to erase or not */
static int WIDTH = 1 ;                  /* pen width */

/* 
 *   buffer variables (actual buffer is Tcl global var CoordList)
 */
static float LASTX = 0. ;
static float LASTY = 0. ;
static int LISTEMPTY = 1; /* 0 if there are coordinates stored in the list */

void flushbuffer(int COLOR) { /* used to buffer create line commands */ 
  if (interpreter != NULL)
    {
      if (LISTEMPTY == 0) /* flush buffer */
	{
	  char rgb[100];
	  char width[10];
	  /* set background color */
	  sprintf(rgb, "#%2.2x%2.2x%2.2x", 
		  R_table[COLOR],G_table[COLOR],B_table[COLOR]);
	  sprintf(width, "%d", WIDTH);
	  Tcl_VarEval(interpreter, "eval ",canvasid, 
		      " create line $CoordList",
		      " -fill ",rgb,
		      " -width ",width," -capstyle round",
		      (char *) NULL);
	}
      LISTEMPTY = 1;
    }
}


/************************************************************************
 *
 *  tcltkdriv -- prototype PGPLOT Driver  "/TCLTK"
 *
 ************************************************************************/

/*
 *  dev_name -- the PGPLOT device name
 *
 */
static char dev_name[] = "TCLTK" ;

/*
 *  SCALE -- the number of pixels per inch -- guess for now
 */
#define SCALE		92

/*
 *  POLY is used for the polygon fill routine
 */
static long POLY = 0 ;

#if defined(_MSC_VER) /* header for Microsoft VC++ */
extern void __stdcall TCLTKDRIV( FUNC, BUFFER, NBUF, STRING, len, NSTR)
#else /* assumes a length argument at end */
#  ifdef PG_PPU
#    define TCLTKDRIV tcltkdriv_
#  else
#    define TCLTKDRIV tcltkdriv
#  endif
void TCLTKDRIV( FUNC, BUFFER, NBUF, STRING, NSTR, len)
#endif

  int *FUNC ;				/* function */
  float *BUFFER ;      	                /* floating data */
  int *NBUF ;				/* length of BUFFER */
  char *STRING ;       			/* character data */
  int *NSTR ;				/* length of STRING */
  int len ;				/* string length */
{
    int i ;
    double x; 

    *NBUF = 0 ;

    switch( *FUNC ) {
/*
 *************** return device name *************************************
 */
    case 1:
	strcpy( STRING, dev_name ) ;
	*NSTR = strlen( dev_name ) ;
	for( i = *NSTR; i < len ; i++ ) /* Blank out string */
	    STRING[i] = ' ' ;
	break ;
/*
 *************** return minimum range of view surface and color index ***
 */
    case 2:
	BUFFER[0] = 0 ;			/* minimum X value */
	BUFFER[1] = X_WID - 1 ;		/* maximum X value */
	BUFFER[2] = 0 ;			/* minimum Y value */
	BUFFER[3] = Y_WID - 1 ;		/* maximum Y value */
	BUFFER[4] = 0 ;			/* minimum color value */
	BUFFER[5] = NCOLOR - 1 ;	/* maximum color value */
	*NSTR = 0 ;
	*NBUF = 6 ;
	break ;
/*
 *************** return device scale ************************************
 */
    case 3:
	BUFFER[0] = SCALE ;		/* X units per inch */
	BUFFER[1] = SCALE ;		/* Y units per inch */
/*
 *  The true pen with is, of course, 1 pixel, or 1.0 in device
 *  coordinates, but this fails miserably for PGPLOT.  A smaller
 *  pen width helps somewhat, but I have not twiddled this parameter
 *  enough to find the best value.  the /CGI interface has similar
 *  problems.
 *
 */
					/* should really be 1.0 */
	BUFFER[2] = 0.7 ;		/* pen width */
	*NSTR = 0 ;
	*NBUF = 3 ;
	break ;
/*
 *************** return device capabilities *****************************
 */
    case 4:
	STRING[0] = 'I' ;		/* interactive device */
	STRING[1] = 'C' ;		/* cursor is available */
	STRING[2] = 'N' ;		/* no dashed lines */
	STRING[3] = 'A' ;		/* polygon fill available */
	STRING[4] = 'T' ;		/* support for fat lines */
	STRING[5] = 'R' ;		/* rectangle fill available */
	STRING[6] = 'N' ;		/* no pixel commands */
        STRING[7] = 'N' ;		/* image not lost on exit */
	STRING[8] = 'N' ;		/* accepts color rep. queries */
	STRING[9] = 'N' ;		/* no opcode 28 */
	*NSTR = 10 ;
	break ;
/*
 *************** return default device/file name ************************
 */
    case 5:
      strcpy( STRING, ".pgplot" ) ;     /* default name */
      *NSTR = 7 ;
      break ;
/*
 *************** return default size of view ****************************
 */
    case 6:
	BUFFER[0] = 0 ;			/* default X min */
	BUFFER[1] = X_WID - 1 ;		/* default X max */
	BUFFER[2] = 0 ;			/* default Y min */
	BUFFER[3] = Y_WID - 1 ;		/* default Y max */
	*NSTR = 0 ;
	*NBUF = 4 ;
	break ;
/*
 *************** return miscellaneous defaults **************************
 */
    case 7:
	BUFFER[0] = 2.0 ;		/* return a random number */
	*NBUF = 1 ;
	*NSTR = 0 ;
	break ;
/*
 *************** select device ******************************************
 */
    case 8:
	/* do nothing */
	break ;
/*
 *************** open workstation ***************************************
 */
    case 9:
	BUFFER[0] = 0.0 ;		/* return channel 0 */
	BUFFER[1] = 1.0 ;		/* successful open */
	if( BUFFER[2] )			/* no-erase mode */
	    APPEND = 1 ;
	else
	    APPEND = 0 ;
	*NBUF = 2 ;
	if (*NSTR > 1) {
	  for (i=0 ; i < *NSTR; i++) canvasid[i] = STRING[i];
	  canvasid[*NSTR] = 0 ;		/* terminate the string */
	} else {
	  strcpy( canvasid, ".pgplot") ;
	}
	/* create the widget if it does not exist */
	Tcl_VarEval(interpreter," catch {pack [canvas ", canvasid, "] }",
		    (char *) NULL);
	/* define bindings needed for cursor input */
	Tcl_VarEval(interpreter," bind ", canvasid, " <Button-1> ",
		    "{set xcur \"%x\"; set ycur \"%y\"; set chcur A}",
		    (char *) NULL);
	Tcl_VarEval(interpreter," bind ", canvasid, " <Button-2> ",
		    "{set xcur \"%x\"; set ycur \"%y\"; set chcur D}",
		    (char *) NULL);
	Tcl_VarEval(interpreter," bind ", canvasid, " <Button-3> ",
		    "{set xcur \"%x\"; set ycur \"%y\"; set chcur X}",
		    (char *) NULL);
	Tcl_VarEval(interpreter," bind ", canvasid, " <Any-Key> ",
		    "{set xcur \"%x\"; set ycur \"%y\"; set chcur \"%A\"}",
		    (char *) NULL);
	/* get the size of the widget */
	if (TCL_ERROR == Tcl_VarEval(interpreter, canvasid," cget -width ", 
				     (char *) NULL)) {
	  Tcl_VarEval(interpreter, " puts {", 
		      "\n Error obtaining size of canvas: ",
		      canvasid,"\n}",
		      (char *) NULL);
	  BUFFER[1] = 0.0 ;		/* unsuccessful open */
	  break ;
	} 
	
	Tcl_GetDouble(interpreter, interpreter->result, &x);
	X_WID = x;
	if (TCL_ERROR == Tcl_VarEval(interpreter, canvasid," cget -height", 
				     (char *) NULL)) {
	  Tcl_VarEval(interpreter, " puts {", 
		      "\n Error obtaining size of canvas: ",
		      canvasid,"\n}",
		      (char *) NULL);
	  BUFFER[1] = 0.0 ;		/* unsuccessful open */
	  break ;
	}
	Tcl_GetDouble(interpreter, interpreter->result, &x);
	Y_WID = x;
	*NSTR = 0 ;
	break ;
/*
 *************** close workstation **************************************
 */
    case 10:
      flushbuffer(COLOR); /* flush buffer */
      if (interpreter != NULL)
	Tcl_Eval(interpreter, "update"); /* update the screen */
      *NSTR = 0 ;
      break ;
/*
 *************** begin picture ******************************************
 */
    case 11:
	if( !APPEND )	
	  if (interpreter != NULL)		/* erase screen */
	    {
	      char rgbstr[9];
	      Tcl_VarEval(interpreter, canvasid," delete all", (char *) NULL);

	      /* set background color */
	      sprintf(rgbstr, "#%2.2x%2.2x%2.2x", 
		      R_table[0],G_table[0],B_table[0]);
	      Tcl_VarEval(interpreter, canvasid," config -bg ", rgbstr, 
			  (char *) NULL);
	    }
	LISTEMPTY = 1; /* empty buffer */
	*NSTR = 0 ;
	break ;
/*
 *************** draw line **********************************************
 */
    case 12:
      if (interpreter != NULL)
	{
	  char cmd[100];
	  if (LISTEMPTY == 0 && /* can we append to the existing list? */
	      LASTX == BUFFER[0] && LASTY == BUFFER[1])
	    {
	      sprintf(cmd, " %.2f %.2f", BUFFER[2], Y_WID - BUFFER[3]);
	      Tcl_SetVar(interpreter, "CoordList", cmd, TCL_APPEND_VALUE);
	    }
	  else if (LISTEMPTY == 0) /* no */
	    flushbuffer(COLOR); /* flush buffer */

	  if (LISTEMPTY == 1) /* no previous buffered lines */
	    {
	      sprintf(cmd, "%.2f %.2f %.2f %.2f",
		  BUFFER[0], Y_WID - BUFFER[1], BUFFER[2], Y_WID - BUFFER[3]);
	      Tcl_SetVar(interpreter, "CoordList", cmd, 0);
	      LISTEMPTY = 0;
	    }
	  LASTX = BUFFER[2] ;
	  LASTY = BUFFER[3];
	}
      *NSTR = 0 ;
      break ;
/*
 *************** draw dot ***********************************************
 */
    case 13:
      flushbuffer(COLOR); /* flush buffer */
      if (interpreter != NULL)
	{
	  char cmd[100];
	  sprintf(cmd, "%.2f %.2f %.2f %.2f",
		  BUFFER[0], Y_WID - BUFFER[1], 
		  BUFFER[0]+1, Y_WID - BUFFER[1]+1);
	  Tcl_SetVar(interpreter, "CoordList", cmd, 0);
	  flushbuffer(COLOR); /* flush buffer */
	}
      *NSTR = 0 ;
      break ;
/*
 *************** end picture ********************************************
 */
    case 14:
      if (interpreter != NULL) {
	if( BUFFER[0] ) { /* clear screen */
	  char rgbstr[9];
	  Tcl_VarEval(interpreter, canvasid," delete all", (char *) NULL);
	  /* set background color */
	  sprintf(rgbstr, "#%2.2x%2.2x%2.2x",
		  R_table[0],G_table[0],B_table[0]);
	  Tcl_VarEval(interpreter, canvasid," config -bg ", rgbstr, 
		      (char *) NULL);
	}
	else 
	  flushbuffer(COLOR); /* flush buffer */

	Tcl_Eval(interpreter, "update"); /* update the screen */
      }
      LISTEMPTY = 1; /* empty buffer */
      *NSTR = 0 ;
      break ;
/*
 *************** set color index ****************************************
 */
    case 15:
      flushbuffer(COLOR); /* flush buffer */
      COLOR = BUFFER[0];
      *NSTR = 0 ;
      break ;
/*
 *************** flush buffer *******************************************
 */
    case 16:
      flushbuffer(COLOR); /* flush buffer */
      if (interpreter != NULL)
	Tcl_Eval(interpreter, "update"); /* update the screen */
      *NSTR = 0 ;
      break ;
/*
 *************** read cursor ********************************************
 */
    case 17:
      if (interpreter != NULL)
	*NSTR = 0 ;
	{
	  flushbuffer(COLOR); /* flush buffer */

	  Tcl_Eval(interpreter, "update"); /* update the screen */

	  /* no rubber-band lines, for now at least */
	  Tcl_VarEval(interpreter," focus ",canvasid,
		      "; tkwait variable xcur; focus",
		      (char *) NULL);
	  {
	    char * val;
	    double x;
	    val = Tcl_GetVar(interpreter, "xcur", 0);
	    Tcl_GetDouble(interpreter, val, &x);
	    BUFFER[0] = x ;
	    val = Tcl_GetVar(interpreter, "ycur", 0);
	    Tcl_GetDouble(interpreter, val, &x);
	    BUFFER[1] = Y_WID - x ;
	    val = Tcl_GetVar(interpreter, "chcur", 0);
	    STRING[0] = val[0];
	    *NBUF = 2 ;
	    *NSTR = 1 ;
	  }
	}
      break ;
/*
 *************** erase alpha screen *************************************
 */
    case 18:
	*NSTR = 0 ;
	/* no alpha screen to erase */
	break ;
/*
 *************** set line style *****************************************
 */
    case 19:
	*NSTR = 0 ;
	break ;
/*
 *************** polygon fill *******************************************
 */
    case 20:
      *NSTR = 0 ;
      flushbuffer(COLOR); /* flush buffer */
      if( POLY == 0 ) {
	POLY = BUFFER[0] ;
	Tcl_SetVar(interpreter, "CoordList", " ", 0);
      }
      else {
	char cmd[100];
	sprintf(cmd, " %.2f %.2f",
		BUFFER[0], Y_WID - BUFFER[1]);
	Tcl_SetVar(interpreter, "CoordList", cmd, TCL_APPEND_VALUE);
	POLY--;
	if( POLY == 0 && interpreter != NULL) {
	  /* set background color */
	  sprintf(cmd, "#%2.2x%2.2x%2.2x", 
		  R_table[COLOR],G_table[COLOR],B_table[COLOR]);
	  Tcl_VarEval(interpreter, "eval ",canvasid, 
		      " create polygon $CoordList",
		      " -fill ",cmd,
		      (char *) NULL);
	}
      }
      break ;
/*
 *************** set color represention *********************************
 */
    case 21:
	i = BUFFER[0] ;
	R_table[i] = 255.9 * BUFFER[1] ;
	G_table[i] = 255.9 * BUFFER[2] ;
	B_table[i] = 255.9 * BUFFER[3] ;
	if (i == 0) { /* BACKGROUND: redefine in widget */
	  char rgbstr[9];
	  /* set background color */
	  sprintf(rgbstr, "#%2.2x%2.2x%2.2x", 
		  R_table[0],G_table[0],B_table[0]);
	  Tcl_VarEval(interpreter, canvasid," config -bg ", rgbstr, 
		      (char *) NULL);
	}
	*NBUF = 4 ;
	break ;
/*
 *************** set line width *****************************************
 */
    case 22:
	*NSTR = 0 ;
	WIDTH = BUFFER[0] + 0.5 ;
	break ;
/*
 *************** escape function ****************************************
 */
    case 23:
	/* no escape functions */
	*NSTR = 0 ;
	break ;
/*
 *************** rectangle fill *****************************************
 */
      case 24:
	*NSTR = 0 ;
	flushbuffer(COLOR); /* flush buffer */
      if (interpreter != NULL)
	{
	  char cmd[100];
	  char rgbstr[9];
	  sprintf(cmd, "%.2f %.2f %.2f %.2f",
		  BUFFER[0], Y_WID - BUFFER[1], 
		  BUFFER[2], Y_WID - BUFFER[3]);
	  sprintf(rgbstr, "#%2.2x%2.2x%2.2x",
		  R_table[COLOR],G_table[COLOR],B_table[COLOR]);
	  Tcl_VarEval(interpreter, canvasid, " create rectangle ", cmd, 
		      " -fill ",rgbstr, 
		      (char *) NULL);
	}
	break ;
/*
 *************** query color representation *****************************
 */
    case 29:
	*NSTR = 0 ;
	i = BUFFER[0] ;
	BUFFER[1] = R_table[i] / 255. ;
	BUFFER[2] = G_table[i] / 255. ;
	BUFFER[3] = B_table[i] / 255. ;
	break ;
/*
 *************** future unknown functions *******************************
 */
    default:
      *NSTR = 0 ;
      *NBUF = -1 ;
      break ;
    }
}

/* replace the standard GRPROM subroutine with a Tk dialog */
#if defined(_MSC_VER)
extern void __stdcall GRPROM (void) 
#else
void  grprom_ (void) 
#endif
{
  Tcl_Eval(interpreter, 
	   "tk_dialog .dialog Pause {Press OK to Continue} {} -1 OK");
}

void NrWarning( char *psWarningText )
{
  /* Print the warning message */

/*  fprintf( stdout, "%s \n", psWarningText );*/
  Tcl_VarEval(interpreter, 
	   "tk_dialog .dialog Warning {", psWarningText  , "} {} -1 OK",
		      (char *) NULL);
}

